;; Copyright (c) Iwata <iwata@quasiquote.org>
;;
;; Permission is hereby granted, free of charge, to any person obtaining
;; a copy of this software and associated documentation files (the
;; "Software"), to deal in the Software without restriction, including
;; without limitation the rights to use, copy, modify, merge, publish,
;; distribute, sublicense, and/or sell copies of the Software, and to
;; permit persons to whom the Software is furnished to do so, subject to
;; the following conditions:
;;
;; The above copyright notice and this permission notice shall be included
;; in all copies or substantial portions of the Software.
;;
;; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
;; EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
;; MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
;; IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
;; CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
;; TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
;; SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

(declare
 (unit twitter-core)
 (export
  get-friends-timeline
  get-profile-image-path-from-url
  get-profile-image
  ))

(include "twitter-vars.scm")

(use srfi-1)
(use utils)
(use http-client)
(use ssax)
(use sxml-tools)
(use uri)

(define-constant twitter-friends-timeline "http://twitter.com/statuses/friends_timeline")

(define (string->encoded-url str)
  (uri-encode-split-path (cdr (uri-split-path str))))

(define (url->sxml url)
  (let ((str (handle-exceptions
              ex
              #f
              (http:GET (string->encoded-url url)))))
    (if str
        (handle-exceptions
         ex
         #f
         (call-with-input-string
          str
          (lambda (port)
            (SSAX:XML->SXML port '())))))))

(define (get-status sxml)
  ((sxpath "/statuses/status") sxml))

(define (get-id-from-status status)
  (if status
      (cadr ((if-car-sxpath "/id") status))
      #f))
(define (get-user-from-status status)
  (if status
      (cadr ((if-car-sxpath "/user/screen_name") status))
      #f))
(define (get-text-from-status status)
  (if status
      (cadr ((if-car-sxpath "/text") status))
      #f))
(define (get-profile-image-from-status status)
  (if status
      (cadr ((if-car-sxpath "/user/profile_image_url") status))
      #f))

(define (get-friends-timeline user)
  (let ((sxml (url->sxml (string-append twitter-friends-timeline "/" user ".xml"))))
    (if sxml
        (let loop ((status (get-status sxml)) (rest '()))
          (if (or (eq? status #f)
                  (null? status))
              (reverse rest)
              (let ((id    (get-id-from-status status))
                    (user  (get-user-from-status status))
                    (text  (get-text-from-status status))
                    (image (get-profile-image-from-status status)))
                (if (and id user text image)
                    (loop (cdr status) (cons (make-twitter-result id user text image) rest))
                    (reverse rest)))))
        #f)))


(define (get-image-path)
  (string-append (get-config-path) "/images"))

(define (get-profile-image-path-from-url url user)
  (let ((ext (pathname-extension url)))
    (if ext
        (make-pathname (string-append (get-image-path) "/") user ext)
        (make-pathname (string-append (get-image-path) "/") user))))

(define (write-image img path)
  (call-with-output-file path
    (lambda (port)
      (display img port))))

(define (get-profile-image user image-url)
  (let ((image (handle-exceptions
                ex
                #f
                (http:GET (string->encoded-url image-url)))))
    (http:close-all-connections!)
    (if image
        (write-image image (get-profile-image-path-from-url image-url user))
        #f)))

(define (get-now) ((foreign-safe-lambda integer "time" c-pointer) #f))

(define-constant day-name   '("Sun" "Mon" "Tue" "Wed" "Thu" "Fri" "Sat"))
(define-constant month-name '("Jan" "Feb" "Mar" "Apr" "May" "Jun" "Jul" "Aug" "Sep" "Oct" "Nov" "Dec"))

(define (get-rfc2822-string)
  (let* ((now (seconds->local-time (get-now)))
         (seconds (vector-ref now 0))  ; the number of seconds after the minute (0 - 59)
         (minutes (vector-ref now 1))  ; the number of minutes after the hour (0 - 59)
         (hours   (vector-ref now 2))  ; the number of hours past midnight (0 - 23)
         (mday    (vector-ref now 3))  ; the day of the month (1 - 31)
         (month   (vector-ref now 4))  ; the number of months since january (0 - 11)
         (year    (vector-ref now 5))  ; the number of years since 1900
         (wday    (vector-ref now 6))  ; the number of days since Sunday (0 - 6)
         (yday    (vector-ref now 7))) ; the number of days since January 1 (0 - 365)
    (format "~a, ~a ~a ~a ~a:~a:~a ~a"
            (list-ref day-name wday)
            mday
            (list-ref month-name month)
            (+ 1900 year)
            hours
            minutes
            seconds
            (local-timezone-abbreviation))))
