;; Copyright (c) Iwata <iwata@quasiquote.org>
;;
;; Permission is hereby granted, free of charge, to any person obtaining
;; a copy of this software and associated documentation files (the
;; "Software"), to deal in the Software without restriction, including
;; without limitation the rights to use, copy, modify, merge, publish,
;; distribute, sublicense, and/or sell copies of the Software, and to
;; permit persons to whom the Software is furnished to do so, subject to
;; the following conditions:
;;
;; The above copyright notice and this permission notice shall be included
;; in all copies or substantial portions of the Software.
;;
;; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
;; EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
;; MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
;; IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
;; CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
;; TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
;; SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

(declare
 (uses twitter-config twitter-libnotify twitter-core))

(use srfi-1)
(use posix)
(use regex)
(use http-client)

(include "twitter-vars.scm")

(define body-escape-regexp (regexp "&[^(amp;|lt;|gt;|quot;)]"))
(define http-regexp (regexp "(s?https?://[-_.!~*'()a-zA-Z0-9;/?:@&=+$,%#]+)"))
(define user-regexp (regexp "@([0-9a-zA-Z_.]+)"))

(define (notify-text-markup str)
  (string-substitute* str `((,body-escape-regexp . "&amp;")
                            (,http-regexp        . "<a href=\"\\1\">\\1</a>")
                            (,user-regexp        . "<a href=\"https://twitter.com/\\1\">@\\1</a>"))))

(define (send-notify config timeout name msg image)
  (let* ((notify-msg  (notify-text-markup msg))
         (n (notify-notification-new name notify-msg image #f)))
    (notify-notification-timeout! n timeout)
    (notify-notification-show n #f)
    (notify-notification-delete! n)))

;; return timeout or #f
(define (apply-notify-filter content rules)
  (cond ((null? rules) ;; always match
         -1)
        (else
         (let ((matched-rule (find (lambda (rule)
                                     (cond ((eq? (car rule) '*) ;; match all
                                            #t)
                                           ((eq? (car rule) 'name)
                                            (string-match (cadr rule) (name-of content)))
                                           ((eq? (car rule) 'text)
                                            (string-match (cadr rule) (text-of content)))
                                           (else
                                            #f)))
                                   rules)))
           (if matched-rule
               (cddr matched-rule)
               #f)))))

(define (main)
  (let* ((config (store-config (read-config)))
         (proxy (config-proxy? config)))
    (if proxy
        (http:add-proxy! (car proxy) (cdr proxy)))
    (let loop ((old-ids '()))
      (let* ((ret (get-friends-timeline (config-name? config)))
             (new-ids (if ret
                          (map (lambda (x)
                                 (if (not (find (lambda (q) (equal? q (id-of x))) old-ids))
                                     (let ((timeout (apply-notify-filter x (config-filter? config))))
                                       (if timeout
                                           (begin
                                             (get-profile-image (name-of x) (image-of x))
                                             (send-notify config timeout
                                                          (name-of x)
                                                          (text-of x)
                                                          (get-profile-image-path-from-url (image-of x) (name-of x)))))))
                                 (id-of x))
                               (reverse ret))
                          old-ids)))
        (sleep (config-interval? config))
        (loop new-ids)))))

(notify-init "twitter-notifier")
(main)
(notify-uninit)


